"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LogoutController = void 0;
const tslib_1 = require("tslib");
const context_1 = require("@loopback/context");
const repository_1 = require("@loopback/repository");
const rest_1 = require("@loopback/rest");
const core_1 = require("@sourceloop/core");
const base_64_1 = require("base-64");
const loopback4_authentication_1 = require("loopback4-authentication");
const loopback4_authorization_1 = require("loopback4-authorization");
const fetch = tslib_1.__importStar(require("node-fetch"));
const url_1 = require("url");
const models_1 = require("../../models");
const repositories_1 = require("../../repositories");
let LogoutController = class LogoutController {
    constructor(req, revokedTokens, refreshTokenRepo, logger) {
        this.req = req;
        this.revokedTokens = revokedTokens;
        this.refreshTokenRepo = refreshTokenRepo;
        this.logger = logger;
    }
    async logout(auth, req) {
        const token = auth === null || auth === void 0 ? void 0 : auth.replace(/bearer /i, '');
        if (!token || !req.refreshToken) {
            throw new rest_1.HttpErrors.UnprocessableEntity("TokenMissing" /* TokenMissing */);
        }
        const refreshTokenModel = await this.refreshTokenRepo.get(req.refreshToken);
        if (!refreshTokenModel) {
            throw new rest_1.HttpErrors.Unauthorized("Token Expired" /* TokenExpired */);
        }
        if (refreshTokenModel.accessToken !== token) {
            throw new rest_1.HttpErrors.Unauthorized("Token Invalid" /* TokenInvalid */);
        }
        await this.revokedTokens.set(token, { token });
        await this.refreshTokenRepo.delete(req.refreshToken);
        if (refreshTokenModel.pubnubToken) {
            await this.refreshTokenRepo.delete(refreshTokenModel.pubnubToken);
        }
        return new core_1.SuccessResponse({
            success: true,
            key: refreshTokenModel.userId,
        });
    }
    async keycloakLogout(auth, req) {
        const token = auth === null || auth === void 0 ? void 0 : auth.replace(/bearer /i, '');
        if (!token || !req.refreshToken) {
            throw new rest_1.HttpErrors.UnprocessableEntity("TokenMissing" /* TokenMissing */);
        }
        const refreshTokenModel = await this.refreshTokenRepo.get(req.refreshToken);
        if (!refreshTokenModel) {
            throw new rest_1.HttpErrors.Unauthorized("Token Expired" /* TokenExpired */);
        }
        if (refreshTokenModel.externalRefreshToken) {
            const params = new url_1.URLSearchParams();
            const logoutUrl = `${process.env.KEYCLOAK_HOST}/auth/realms/${process.env.KEYCLOAK_REALM}/protocol/openid-connect/logout`;
            params.append('refresh_token', refreshTokenModel.externalRefreshToken);
            const strToEncode = `${process.env.KEYCLOAK_CLIENT_ID}:${process.env.KEYCLOAK_CLIENT_SECRET}`;
            fetch
                .default(logoutUrl, {
                method: 'post',
                body: params,
                headers: {
                    Authorization: `Basic ${base_64_1.encode(strToEncode)}`,
                },
            })
                .then(() => {
                this.logger.info(`User ${refreshTokenModel.username} logged off successfully from keycloak.`);
            })
                .catch(err => {
                this.logger.error(`Error while logging off from keycloak. Error :: ${err} ${JSON.stringify(err)}`);
            });
        }
        if (refreshTokenModel.accessToken !== token) {
            throw new rest_1.HttpErrors.Unauthorized("Token Invalid" /* TokenInvalid */);
        }
        await this.revokedTokens.set(token, { token });
        await this.refreshTokenRepo.delete(req.refreshToken);
        if (refreshTokenModel.pubnubToken) {
            await this.refreshTokenRepo.delete(refreshTokenModel.pubnubToken);
        }
        return new core_1.SuccessResponse({
            success: true,
            key: refreshTokenModel.userId,
        });
    }
};
tslib_1.__decorate([
    loopback4_authentication_1.authenticate("bearer" /* BEARER */, {
        passReqToCallback: true,
    }),
    loopback4_authorization_1.authorize({ permissions: ['*'] }),
    rest_1.post('/logout', {
        security: core_1.OPERATION_SECURITY_SPEC,
        description: 'To logout',
        responses: {
            [200 /* OK */]: {
                description: 'Success Response',
                content: {
                    [core_1.CONTENT_TYPE.JSON]: {
                        schema: { 'x-ts-type': core_1.SuccessResponse },
                    },
                },
            },
            ...core_1.ErrorCodes,
        },
    }),
    tslib_1.__param(0, rest_1.param.header.string('Authorization', {
        description: 'This is the access token which is required to authenticate user.',
    })),
    tslib_1.__param(1, rest_1.requestBody({
        content: {
            [core_1.CONTENT_TYPE.JSON]: {
                schema: rest_1.getModelSchemaRef(models_1.RefreshTokenRequest, {
                    partial: true,
                }),
            },
        },
    })),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [String, models_1.RefreshTokenRequest]),
    tslib_1.__metadata("design:returntype", Promise)
], LogoutController.prototype, "logout", null);
tslib_1.__decorate([
    loopback4_authentication_1.authenticate("bearer" /* BEARER */, {
        passReqToCallback: true,
    }),
    loopback4_authorization_1.authorize({ permissions: ['*'] }),
    rest_1.post('/keycloak/logout', {
        security: core_1.OPERATION_SECURITY_SPEC,
        description: 'This API will log out the user from application as well as keycloak',
        responses: {
            [200 /* OK */]: {
                description: 'Success Response',
                content: {
                    [core_1.CONTENT_TYPE.JSON]: {
                        schema: { 'x-ts-type': core_1.SuccessResponse },
                    },
                },
            },
            ...core_1.ErrorCodes,
        },
    }),
    tslib_1.__param(0, rest_1.param.header.string('Authorization', {
        description: 'This is the access token which is required to authenticate user.',
    })),
    tslib_1.__param(1, rest_1.requestBody({
        content: {
            [core_1.CONTENT_TYPE.JSON]: {
                schema: rest_1.getModelSchemaRef(models_1.RefreshTokenRequest, {
                    partial: true,
                }),
            },
        },
    })),
    tslib_1.__metadata("design:type", Function),
    tslib_1.__metadata("design:paramtypes", [String, models_1.RefreshTokenRequest]),
    tslib_1.__metadata("design:returntype", Promise)
], LogoutController.prototype, "keycloakLogout", null);
LogoutController = tslib_1.__decorate([
    tslib_1.__param(0, context_1.inject(rest_1.RestBindings.Http.REQUEST)),
    tslib_1.__param(1, repository_1.repository(repositories_1.RevokedTokenRepository)),
    tslib_1.__param(2, repository_1.repository(repositories_1.RefreshTokenRepository)),
    tslib_1.__param(3, context_1.inject(core_1.LOGGER.LOGGER_INJECT)),
    tslib_1.__metadata("design:paramtypes", [Object, repositories_1.RevokedTokenRepository,
        repositories_1.RefreshTokenRepository, Object])
], LogoutController);
exports.LogoutController = LogoutController;
//# sourceMappingURL=logout.controller.js.map